@@ -2,8 +2,9 @@
 Name               : config.c
 Author             : Marvin Raaijmakers
 Description        : Reads and writes the configuration files
-Date of last change: 15-Oct-2006
-History            : 15-Oct-2006 Modified read_preferences() and
+Date of last change: 28-Jun-2007
+History            : 28-Jun-2007 Fixed compiler warnings
+                     15-Oct-2006 Modified read_preferences() and
                                  write_preferences() so that they read/writer the
                                  'mixer_channel' element instead of the
                                  'documents_dir' element.
@@ -31,7 +32,7 @@ History            : 15-Oct-2006 Modified read_preferences() and
                                  reading a key that does appear in the keyboard
                                  configuration file but not in the keyboard file
 
-    Copyright (C) 2005-2006 Marvin Raaijmakers
+    Copyright (C) 2005-2007 Marvin Raaijmakers
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -214,7 +215,6 @@ Desciption:
 	XmlContent	*key_element,
 			*keyname_element,
 			*default_action_element;
-	char		*action_type_string;
 	Boolean		error;
 	
 	*key_settings = NULL;
