@@ -2,8 +2,9 @@
 Name               : set_data.c
 Author             : Marvin Raaijmakers
 Description        : Sets data to the GUI
-Date of last change: 27-May-2006
-History            : 27-May-2006 reload_key_treeview() selects the first row
+Date of last change: 15-Oct-2006
+History            : 15-Oct-2006 Added reload_mixer_channel_combobox()
+                     27-May-2006 reload_key_treeview() selects the first row
                      30-Apr-2006 Modified reload_plugin_function_list(),
                                  reload_plugin_list(), reload_plugin_treeview(),
                                  reload_key_treeview() and reload_current_keyboard()
@@ -39,6 +40,7 @@ History            : 27-May-2006 reload_key_treeview() selects the first row
 #endif
 #include <gtk/gtk.h>
 #include <string.h>
+#include <alsa/asoundlib.h>
 
 #include "callbacks.h"
 #include "interface.h"
@@ -54,7 +56,7 @@ History            : 27-May-2006 reload_key_treeview() selects the first row
 #define GTK_ALIGN_CENTER	(0.5)
 
 #define ONE_COLUMN	(1)
-#define FIRST_COLUMN	(0) 
+#define FIRST_COLUMN	(0)
 #define SORTID_FIRSTCOL	(0)
 
 
@@ -394,7 +396,7 @@ Description:
 	/* Add all plugin names to the list */
 	while (plugin != NULL)
 	{
-		list_append_item (plugin_list_widget, plugil_plugin_get_name(plugin));
+		list_append_item (plugin_list_widget, plugini_get_name(plugin));
 		plugin = plugin->next;
 	}
 	
@@ -427,7 +429,7 @@ Description:
 	/* Add all plugin names to the list */
 	while (plugin != NULL)
 	{
-		list_store_append_string (list_store, plugil_plugin_get_name(plugin));
+		list_store_append_string (list_store, plugini_get_name(plugin));
 		plugin = plugin->next;
 	}
 }
@@ -617,15 +619,14 @@ reload_preferences (	KTGUI_type	*gui,
 Input:
 	preferences	- These preferences must be set to the preferences entries
 Output:
-	gui		- The documents_entry, home_page_entry, search_page_entry,
-			  browser_entry, email_prog_entry and chat_entry members
-			  will contain the corresponding member in 'preferences'.
+	gui		- The home_page_entry, search_page_entry, browser_entry,
+			  email_prog_entry and chat_entry members will contain the
+			  corresponding member in 'preferences'.
 Description:
 	This function sets the information of preferences to all prefecences entries
 	in 'gui'.
 */
 {
-	gtk_entry_set_text (gui->documents_entry, preferences->documents_dir);
 	gtk_entry_set_text (gui->home_page_entry, preferences->home_page);
 	gtk_entry_set_text (gui->search_page_entry, preferences->search_page);
 	gtk_entry_set_text (gui->browser_entry, preferences->browser);
@@ -634,6 +635,59 @@ Description:
 }
 
 
+void
+reload_mixer_channel_combobox (	const char	*current_channel,
+				GtkComboBox	*combobox )
+/*
+Input:
+	current_channel	- The name of the channel that should be selected in
+			  'combobox'.
+	combobox	- A created GtkComboBox.
+Output:
+	combobox	- Will be loaded with all available mixer channels. If there
+			  is a channel with the name equal to 'current_channel',
+			  then this channel will be selected otherwise the first
+			  channel will be selected.
+*/
+{
+	int index, selected_index;
+	snd_mixer_t *handle;
+	snd_mixer_selem_id_t *sid;
+	snd_mixer_elem_t *elem;
+	
+	/* Remove all items (= channels) from 'combobox': */
+	gtk_list_store_clear (GTK_LIST_STORE( gtk_combo_box_get_model(combobox) ));
+	
+	snd_mixer_selem_id_alloca (&sid);
+	if (snd_mixer_open(&handle, 0) >= 0)
+	{
+		if (snd_mixer_attach (handle, "default") >= 0 &&
+		    snd_mixer_selem_register (handle, NULL, NULL) >= 0 &&
+		    snd_mixer_load (handle) >= 0)
+		{
+			selected_index = index = 0;
+			/* For each channel: */
+			for (elem = snd_mixer_first_elem(handle); elem; elem = snd_mixer_elem_next(elem))
+			{
+				if (snd_mixer_selem_is_active( elem ) && snd_mixer_selem_has_playback_volume( elem ))
+				{
+					snd_mixer_selem_get_id (elem, sid);
+					gtk_combo_box_append_text (combobox, snd_mixer_selem_id_get_name( sid ));
+					/* Is this the channel that should be selected? */
+					if (strcmp (snd_mixer_selem_id_get_name( sid ), current_channel) == EQUAL)
+					{
+						selected_index = index;
+					}
+					index++;
+				}
+			}
+			gtk_combo_box_set_active (combobox, selected_index);
+		}
+		snd_mixer_close(handle);
+	}
+}
+
+
 void
 init_main_window (	KTKeyboardName		*current_keyboard_name,
 			KTKeySettingsList	*key_list,
@@ -673,7 +727,6 @@ Description:
 	gui->default_entry = GTK_ENTRY( lookup_widget(gui->main_window, "default_entry") );
 	gui->plugin_combo_entry = GTK_ENTRY( lookup_widget(gui->main_window, "plugin_combo_entry") );
 	gui->plugin_function_combo_entry = GTK_ENTRY( lookup_widget(gui->main_window, "plugin_function_combo_entry") );
-	gui->documents_entry = GTK_ENTRY( lookup_widget(gui->main_window, "documents_entry") );
 	gui->home_page_entry = GTK_ENTRY( lookup_widget(gui->main_window, "home_page_entry") );
 	gui->search_page_entry = GTK_ENTRY( lookup_widget(gui->main_window, "search_page_entry") );
 	gui->browser_entry = GTK_ENTRY( lookup_widget(gui->main_window, "browser_entry") );
@@ -684,6 +737,7 @@ Description:
 	gui->special_action_radiobutton = GTK_RADIO_BUTTON( lookup_widget(gui->main_window, "special_action_radiobutton") );
 	gui->plugin_combo_list = GTK_LIST( GTK_COMBO( lookup_widget(gui->main_window, "plugin_combo") )->list );
 	gui->plugin_function_combo_list = GTK_LIST( GTK_COMBO( lookup_widget(gui->main_window, "plugin_function_combo") )->list );
+	gui->mixer_channel_combobox = GTK_COMBO_BOX( lookup_widget(gui->main_window, "mixer_channel_combobox") );
 	
 	/****** Initialize the treeviews ******/
 	init_plugin_treeview (gui->plugin_treeview);
@@ -698,6 +752,7 @@ Description:
 	reload_current_keyboard (GTK_ENTRY( lookup_widget(gui->main_window, "keyboard_model_entry") ),
 	                         GTK_ENTRY( lookup_widget(gui->main_window, "keyboard_manufacturer_entry") ),
 	                         current_keyboard_name);
+	reload_mixer_channel_combobox (preferences->mixer_channel, gui->mixer_channel_combobox);
 	reload_preferences (gui, preferences);
 	gtk_label_set_text (GTK_LABEL( lookup_widget(GTK_WIDGET(gui->main_window), "version_val_label") ), VERSION);
 }
