@@ -3,10 +3,11 @@ Name               : setkeycodes.c
 Author             : Marvin Raaijmakers
 Description        : Assigns a keycode to every function key of the current
                      keyboard.
-Date of last change: 25-Jun-2006
-History            : 25-Jun-2006 setkeycodes() now skips keys with scancode = 0
+Date of last change: 20-Jul-2007
+History            : 20-Jul-2007 Added support for USB keyboards
+                     25-Jun-2006 setkeycodes() now skips keys with scancode = 0
 
-    Copyright (C) 2005-2006 Marvin Raaijmakers
+    Copyright (C) 2005-2007 Marvin Raaijmakers
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -36,10 +37,12 @@ History            : 25-Jun-2006 setkeycodes() now skips keys with scancode = 0
 #include <keytouch-init.h>
 
 void
-setkeycodes (KTKeySettingsList *key_list)
+setkeycodes (KTKeySettingsList *key_list,
+             Boolean           usb_keyboard)
 /*
 Input:
-	key_list	- The list of keys to set the keycodes for
+	key_list     - The list of keys to set the keycodes for.
+	usb_keyboard - Indicates whether the keyboard is connected via USB.
 Output:
 	-
 Returns:
@@ -47,33 +50,63 @@ Returns:
 Description:
 	For every key in key_list this function sets the keycode to the scancode of
 	the key in kernel scancode-to-keycode mapping table.
+	If 'usb_keyboard' is TRUE and a keys 'usagecode' member is nonzero then this
+	function will first try to set the keycode to the usagecode. If that fails,
+	the keycode will be set to the scancode.
 */
 {
-	KTKeySettings		*key;
-	int			fd;
-	struct kbkeycode	a;
+	KTKeySettings *key;
+	int fd;
+	struct kbkeycode a;
+	Boolean set_to_scancode;
 	
 	fd = getfd(NULL);
 	for (key = key_list->head; key != NULL; key = key->next)
 	{
-		a.keycode = key->keycode;
-		a.scancode = key->scancode;
-		if (a.scancode >= 0xe000)
-		{
-			a.scancode -= 0xe000;
-			a.scancode += 128;	/* some kernels needed +256 */
-		}
-		if (a.keycode >= NUM_KEYCODES)
+		if (key->keycode >= NUM_KEYCODES)
 		{
 			KTError ("Failed to set keycode:", "");
 			fprintf (stderr, "keycode %d is larger than %d\n",
 			         a.keycode, NUM_KEYCODES-1);
 		}
-		else if (a.scancode && ioctl(fd, KDSETKEYCODE, &a)) /* Set the keycode */
+		/*printf ("%d %d %d 0x%x\n", usb_keyboard, key->keycode, key->scancode, key->usagecode);*/
+		
+		a.keycode = key->keycode;
+		
+		if (usb_keyboard && key->usagecode)
 		{
-			KTError ("Failed to set keycode:", "");
-			fprintf (stderr, "keycode %d to scancode %d (0x%x)\n",
-			         a.keycode, a.scancode, a.scancode);
+			a.scancode = key->usagecode;
+			if (ioctl(fd, KDSETKEYCODE, &a)) /* Set the keycode */
+			{
+				KTError ("Failed to set keycode:", "");
+				fprintf (stderr, "keycode %d to usage code 0x%x\n",
+				         a.keycode, a.scancode);
+				set_to_scancode = TRUE;
+			}
+			else
+			{
+				set_to_scancode = FALSE;
+			}
+		}
+		else
+		{
+			set_to_scancode = TRUE;
+		}
+		
+		if (set_to_scancode)
+		{
+			a.scancode = key->scancode;
+			if (a.scancode >= 0xe000)
+			{
+				a.scancode -= 0xe000;
+				a.scancode += 128; /* some kernels needed +256 */
+			}
+			if (a.scancode && ioctl(fd, KDSETKEYCODE, &a)) /* Set the keycode */
+			{
+				KTError ("Failed to set keycode:", "");
+				fprintf (stderr, "keycode %d to scancode %d (0x%x)\n",
+				         a.keycode, a.scancode, a.scancode);
+			}
 		}
 	}
 }
