@@ -2,8 +2,11 @@
 Name               : check_syntax.c
 Author             : Marvin Raaijmakers
 Description        : Checks if the syntax of a keyboard file is valid
-Date of last change: 28-Jun-2007
-History            : 28-Jun-2007 Fixed compiler warnings
+Date of last change: 29-Sep-2007
+History            : 21-Jul-2007 Added check_keyboard_info() and checks for
+                                 'usb-code' elements in check_key_settings()
+                     20-Jul-2007 Fixed memory leak (key_name) in check_key_settings
+                     28-Jun-2007 Fixed compiler warnings
                      17-Sep-2006 Added support for ACPI hotkeys
                      24-Jun-2006 Added checks for keycodes and scancodes
 
@@ -49,6 +52,7 @@ static Boolean check_key_settings (	XmlContent		*keylist_element,
 					XmlElementType		*keyname_type,
 					XmlElementType		*keycode_type,
 					XmlElementType		*scancode_type,
+					XmlElementType		*usbcode_type,
 					XmlElementType		*acpi_event_descr_type,
 					XmlElementType		*default_action_type,
 					XmlElementType		*pluginname_type,
@@ -65,6 +69,7 @@ static Boolean syntax_version_ok (	XmlContent	*file_info_element,
 					unsigned int	version_minor );
 static Boolean check_last_change_date (	XmlContent	*file_info_element,
 					XmlDocument	*document );
+static Boolean check_keyboard_info (XmlDocument *keyboard_file, XmlContent  *keyboard_element);
 
 
 
@@ -121,6 +126,7 @@ check_key_settings (	XmlContent		*keylist_element,
 			XmlElementType		*keyname_type,
 			XmlElementType		*keycode_type,
 			XmlElementType		*scancode_type,
+			XmlElementType		*usbcode_type,
 			XmlElementType		*acpi_event_descr_type,
 			XmlElementType		*default_action_type,
 			XmlElementType		*pluginname_type,
@@ -139,6 +145,7 @@ Input:
 	keyname_type		- The XmlElementType for the key-name element
 	keycode_type		- The XmlElementType for the keycode element
 	scancode_type		- The XmlElementType for the scancode element
+	usbcode_type		- The XmlElementType for the usb-code element
 	acpi_event_descr_type	- The XmlElementType for the event-descr element
 	default_action_type	- The XmlElementType for the default-action element
 	pluginname_type		- The XmlElementType for the plugin-name element
@@ -153,11 +160,11 @@ Input:
 Output:
 	valid			- Will be TRUE if the index'th element of 'key_type'
 				  in 'keylist_element', is valid (otherwise FALSE).
-	num_keys		- If '*valid' is TRUE then 'num_keys' will be
+	num_keys		- If '*valid' is set to TRUE then 'num_keys' will be
 				  increased with 1 if it is smaller than
 				  MAX_NUM_KEYS-1.
-	key_names		- If '*valid' is TRUE then the (*num_key-1)'th (if
-				  *num_keys < MAX_NUM_KEYS) element of this array,
+	key_names		- If '*valid' is set to TRUE then the (*num_key-1)'th
+				  (if *num_keys < MAX_NUM_KEYS) element of this array,
 				  will point to the key name of the checked key
 				  setting. The memory of this name is allocated and
 				  should be freed if it is no longer needed.
@@ -175,6 +182,7 @@ Desciption:
 			*keyname_element,
 			*keycode_element,
 			*scancode_element,
+			*usbcode_element,
 			*default_action_element;
 	unsigned int	i, keycode;
 	char		*key_name,
@@ -190,12 +198,18 @@ Desciption:
 	{
 		key_type_str = XmlGetAttributeValue (key_type_attrname, key_element);
 		is_acpi_hotkey = (key_type_str && !strcmp(key_type_str, ATTR_VAL_ACPIHOTKEY));
-		if ((keyname_element = XmlGetElementOfType(XmlElementContentList(key_element), keyname_type, 0)) == NULL ||
-		    (default_action_element = XmlGetElementOfType(XmlElementContentList(key_element), default_action_type, 0)) == NULL ||
-		    (keycode_element = XmlGetElementOfType(XmlElementContentList(key_element), keycode_type, 0)) == NULL ||
+		
+		keyname_element = XmlGetElementOfType(XmlElementContentList(key_element), keyname_type, 0);
+		default_action_element = XmlGetElementOfType(XmlElementContentList(key_element), default_action_type, 0);
+		keycode_element = XmlGetElementOfType(XmlElementContentList(key_element), keycode_type, 0);
+		event_descr_element = XmlGetElementOfType(XmlElementContentList(key_element), acpi_event_descr_type, 0);
+		scancode_element = XmlGetElementOfType(XmlElementContentList(key_element), scancode_type, 0);
+		usbcode_element = XmlGetElementOfType(XmlElementContentList(key_element), usbcode_type, 0);
+		
+		if (keyname_element == NULL || default_action_element == NULL || keycode_element == NULL ||
 		    (is_acpi_hotkey ?
-		       (event_descr_element = XmlGetElementOfType(XmlElementContentList(key_element), acpi_event_descr_type, 0)) == NULL :
-		       (scancode_element = XmlGetElementOfType(XmlElementContentList(key_element), scancode_type, 0)) == NULL) )
+		      event_descr_element == NULL :
+		      (scancode_element == NULL && usbcode_element == NULL) ))
 		{
 			KTSetErrorMsg (_("The keyboard file contains an incomplete key element."), "");
 			*valid = FALSE;
@@ -228,17 +242,34 @@ Desciption:
 			{
 				if (!is_acpi_hotkey)
 				{
-					/** Check the scancode **/
-					scancode_string = XmlGetElementString(scancode_element, "");
-					/* Convert the string of the scancode to a integer */
-					strtol (scancode_string, &end_pointer, 0);
-					/* If the scancode string is not a vallid scancode */
-					if (*end_pointer != '\0')
+					if (scancode_element)
 					{
-						KTSetErrorMsg (_("'%s' is an invalid scancode."), scancode_string);
-						*valid = FALSE;
+						/** Check the scancode **/
+						scancode_string = XmlGetElementString(scancode_element, "");
+						/* Convert the string of the scancode to a integer */
+						strtol (scancode_string, &end_pointer, 0);
+						/* If the scancode string is not a valid scancode */
+						if (*end_pointer != '\0')
+						{
+							KTSetErrorMsg (_("'%s' is an invalid scancode."), scancode_string);
+							*valid = FALSE;
+						}
+						XmlFree (scancode_string);
+					}
+					if (usbcode_element)
+					{
+						/** Check the USB code **/
+						scancode_string = XmlGetElementString(usbcode_element, "");
+						/* Convert the string of the USB code to a integer */
+						strtol (scancode_string, &end_pointer, 16);
+						/* If the string is not a valid USB code */
+						if (*end_pointer != '\0')
+						{
+							KTSetErrorMsg (_("'%s' is an invalid USB code."), scancode_string);
+							*valid = FALSE;
+						}
+						XmlFree (scancode_string);
 					}
-					XmlFree (scancode_string);
 				}
 				
 				if (*valid)
@@ -272,17 +303,19 @@ Desciption:
 					}
 					XmlFree (keycode_string);
 					
-					if (*valid && *num_keys < MAX_NUM_KEYS-1)
+					if (*valid && *num_keys < MAX_NUM_KEYS)
 					{
 						key_names[*num_keys] = key_name;
 						(*num_keys)++;
 					}
-					else
-					{
-						XmlFree (key_name);
-					}
 				}
 			}
+			/* If the key is invalid or the name could
+			 * not have been added to the name list */
+			if (!*valid || *num_keys >= MAX_NUM_KEYS)
+			{
+				XmlFree (key_name);
+			}
 		}
 	}
 	
@@ -389,6 +422,91 @@ Description:
 }
 
 
+Boolean
+check_keyboard_info (XmlDocument *keyboard_file,
+                     XmlContent  *keyboard_element)
+{
+	XmlContent *keyboard_name_element,
+	           *keyboard_info_element,
+	           *usb_info_element,
+	           *vendor_id_element,
+	           *product_id_element;
+	XmlElementType *keyboard_name_type,
+	               *keyboard_info_type,
+	               *model_type,
+	               *manufacturer_type,
+	               *usb_info_type,
+	               *vendor_id_type,
+	               *product_id_type;
+	char *string, *end_pointer;
+	Boolean error;
+	
+	error = FALSE;
+	if ((keyboard_info_type = XmlGetElementType(ELEM_TYPE_KEYBOARDINFO, keyboard_file, FALSE)) == NULL ||
+	    (keyboard_name_type = XmlGetElementType(ELEM_TYPE_KEYBOARDNAME, keyboard_file, FALSE)) == NULL ||
+	    (model_type = XmlGetElementType(ELEM_TYPE_MODEL, keyboard_file, FALSE)) == NULL ||
+	    (manufacturer_type = XmlGetElementType(ELEM_TYPE_MANUFACTURER, keyboard_file, FALSE)) == NULL ||
+	    (keyboard_info_element =
+	      XmlGetElementOfType (XmlElementContentList(keyboard_element), keyboard_info_type, 0)) == NULL ||
+	    (keyboard_name_element =
+	      XmlGetElementOfType (XmlElementContentList(keyboard_info_element), keyboard_name_type, 0)) == NULL ||
+	    XmlGetElementOfType (XmlElementContentList(keyboard_name_element), model_type, 0) == NULL ||
+	    XmlGetElementOfType (XmlElementContentList(keyboard_name_element), manufacturer_type, 0) == NULL)
+	{
+		KTSetErrorMsg (_("The keyboard file does not provide enough information about the keyboard model."), "");
+		error = TRUE;
+	}
+	else if ((usb_info_type = XmlGetElementType(ELEM_TYPE_USBINFO, keyboard_file, FALSE)) != NULL &&
+	         (usb_info_element =
+	           XmlGetElementOfType (XmlElementContentList(keyboard_info_element), usb_info_type, 0)) != NULL)
+	{
+		if ((vendor_id_type = XmlGetElementType(ELEM_TYPE_VENDORID, keyboard_file, FALSE)) == NULL ||
+		    (vendor_id_element =
+		      XmlGetElementOfType (XmlElementContentList(usb_info_element), vendor_id_type, 0)) == NULL)
+		{
+			KTSetErrorMsg (_("The 'usb-info' element does not contain a '%s' element."), ELEM_TYPE_VENDORID);
+			error = TRUE;
+		}
+		else if ((product_id_type = XmlGetElementType(ELEM_TYPE_PRODUCTID, keyboard_file, FALSE)) == NULL ||
+		         (product_id_element =
+		           XmlGetElementOfType (XmlElementContentList(usb_info_element), product_id_type, 0)) == NULL)
+		{
+			KTSetErrorMsg (_("The 'usb-info' element does not contain a '%s' element."), ELEM_TYPE_PRODUCTID);
+			error = TRUE;
+		}
+		else
+		{
+			string = XmlGetElementString(vendor_id_element, "");
+			/* Convert the string of the vendor ID to a integer */
+			strtol (string, &end_pointer, 16);
+			/* If the string is not a valid vendor ID */
+			if (*end_pointer != '\0')
+			{
+				KTSetErrorMsg (_("'%s' is an invalid vendor ID."), string);
+				error = TRUE;
+			}
+			XmlFree (string);
+			
+			if (!error)
+			{
+				string = XmlGetElementString(product_id_element, "");
+				/* Convert the string of the product ID to a integer */
+				strtol (string, &end_pointer, 16);
+				/* If the string is not a valid product ID */
+				if (*end_pointer != '\0')
+				{
+					KTSetErrorMsg (_("'%s' is an invalid product ID."), string);
+					error = TRUE;
+				}
+				XmlFree (string);
+			}
+		}
+	}
+	
+	return !error;
+}
+
+
 Boolean
 check_keyboard_file (XmlDocument *keyboard_file)
 /*
@@ -405,24 +523,19 @@ Desciption:
 {
 	XmlContent		*keyboard_element,
 				*file_info_element,
-				*keylist_element,
-				*keyboard_name_element,
-				*keyboard_info_element;
+				*keylist_element;
 	XmlElementType		*keyboard_type,
 				*file_info_type,
 				*keylist_type,
 				*key_type,
 				*keyname_type,
 				*scancode_type,
+				*usbcode_type,
 				*event_descr_type,
 				*keycode_type,
 				*default_action_type,
 				*pluginname_type,
-				*pluginfunction_type,
-				*keyboard_name_type,
-				*keyboard_info_type,
-				*model_type,
-				*manufacturer_type;
+				*pluginfunction_type;
 	XmlAttributeName	*action_type_attrname,
 				*key_type_attrname;
 	Boolean			error, valid;
@@ -434,6 +547,7 @@ Desciption:
 	
 	event_descr_type = XmlGetElementType (ELEM_TYPE_ACPI_EVENT_DESCR, keyboard_file, TRUE);
 	scancode_type = XmlGetElementType (ELEM_TYPE_SCANCODE, keyboard_file, TRUE);
+	usbcode_type = XmlGetElementType (ELEM_TYPE_USBCODE, keyboard_file, TRUE);
 	
 	if ((keyboard_type = XmlGetElementType (ELEM_TYPE_KEYBOARD, keyboard_file, FALSE)) == NULL ||
 	    (keyboard_element = XmlGetElementOfType (XmlDocumentContentList(keyboard_file), keyboard_type, 0)) == NULL)
@@ -441,18 +555,9 @@ Desciption:
 		KTSetErrorMsg (_("The keyboard file does not contain a keyboard element."), "");
 		error = TRUE;
 	}
-	else if ((keyboard_info_type = XmlGetElementType(ELEM_TYPE_KEYBOARDINFO, keyboard_file, FALSE)) == NULL ||
-	         (keyboard_name_type = XmlGetElementType(ELEM_TYPE_KEYBOARDNAME, keyboard_file, FALSE)) == NULL ||
-	         (model_type = XmlGetElementType(ELEM_TYPE_MODEL, keyboard_file, FALSE)) == NULL ||
-	         (manufacturer_type = XmlGetElementType(ELEM_TYPE_MANUFACTURER, keyboard_file, FALSE)) == NULL ||
-	         (keyboard_info_element =
-	           XmlGetElementOfType (XmlElementContentList(keyboard_element), keyboard_info_type, 0)) == NULL ||
-	         (keyboard_name_element =
-	           XmlGetElementOfType (XmlElementContentList(keyboard_info_element), keyboard_name_type, 0)) == NULL ||
-	         XmlGetElementOfType (XmlElementContentList(keyboard_name_element), model_type, 0) == NULL ||
-	         XmlGetElementOfType (XmlElementContentList(keyboard_name_element), manufacturer_type, 0) == NULL)
+	else if (!check_keyboard_info (keyboard_file, keyboard_element))
 	{
-		KTSetErrorMsg (_("The keyboard file does provide enough information about the keyboard model."), "");
+		/* NOTE: KTSetErrorMsg() is already called by check_keyboard_info() */
 		error = TRUE;
 	}
 	else if ((keylist_type = XmlGetElementType(ELEM_TYPE_KEYLIST, keyboard_file, FALSE)) == NULL ||
@@ -512,7 +617,7 @@ Desciption:
 		num_keys = 0;
 		for (index = 0;
 		     check_key_settings (keylist_element, index, key_type, keyname_type, keycode_type,
-		                         scancode_type, event_descr_type, default_action_type,
+		                         scancode_type, usbcode_type, event_descr_type, default_action_type,
 		                         pluginname_type, pluginfunction_type, action_type_attrname,
 		                         key_type_attrname, &num_keys, key_names, keycode_used, &valid);
 		     index++)
