@@ -3,9 +3,12 @@ Name               : config_element_types.h
 Author             : Marvin Raaijmakers
 Description        : Contains all ellement types, attribute names and attribute
                      values that appear in the (XML) configuration files.
-Date of last change: 19-Aug-2005
+Date of last change: 21-Jul-2007
+History            : 21-Jul-2007 Added ELEM_TYPE_USBCODE
+                     15-Oct-2006 Replaced ELEM_TYPE_DOCUMENTSDIR by
+                                 ELEM_TYPE_MIXERCHANNEL
 
-    Copyright (C) 2005 Marvin Raaijmakers
+    Copyright (C) 2005-2007 Marvin Raaijmakers
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -24,6 +27,8 @@ Date of last change: 19-Aug-2005
     You can contact me at: marvinr(at)users(dot)sf(dot)net
     (replace (at) by @ and (dot) by .)
 -----------------------------------------------------------------------------------*/
+#ifndef CONFIG_ELEMENT_TYPES_H
+#define CONFIG_ELEMENT_TYPES_H
 
 /* All used attribute names in the configuration files */
 #define ATTR_NAME_ACTIONTYPE		"action-type"
@@ -44,11 +49,12 @@ Date of last change: 19-Aug-2005
 #define ELEM_TYPE_KEYNAME		"name"
 #define ELEM_TYPE_KEYCODE		"keycode"
 #define ELEM_TYPE_SCANCODE		"scancode"
+#define ELEM_TYPE_USBCODE		"usb-code"
 #define ELEM_TYPE_PLUGINNAME		"plugin-name"
 #define ELEM_TYPE_PLUGINFUNCTION	"plugin-function"
 #define ELEM_TYPE_ACTION		"action"
 #define ELEM_TYPE_PREFERENCES		"preferences"
-#define ELEM_TYPE_DOCUMENTSDIR		"documents-dir"
+#define ELEM_TYPE_MIXERCHANNEL		"mixer-channel"
 #define ELEM_TYPE_HOMEPAGE		"home-page"
 #define ELEM_TYPE_SEARCHPAGE		"search-page"
 #define ELEM_TYPE_BROWSER		"browser"
@@ -64,3 +70,8 @@ Date of last change: 19-Aug-2005
 #define ELEM_TYPE_SYNTAXVERSION		"syntax-version"
 #define ELEM_TYPE_LASTCHANGE		"last-change"
 #define ELEM_TYPE_ACPI_EVENT_DESCR	"event-descr"
+#define ELEM_TYPE_USBINFO		"usb-info"
+#define ELEM_TYPE_VENDORID		"vendor-id"
+#define ELEM_TYPE_PRODUCTID		"product-id"
+
+#endif
