@@ -2,8 +2,12 @@
 Name               : set_data.c
 Author             : Marvin Raaijmakers
 Description        : Sets data to the GUI
-Date of last change: 28-Jun-2007
-History            : 28-Jun-2007 Fixed compiler warnings
+Date of last change: 04-Jul-2007
+History            : 13-Jul-2007 Modified keyboard treeview so that it now has a
+                                 tree store instead of a list store
+                     04-Jul-2007 A keyboard/row will be selected by
+                                 reload_keyboard_treeview()
+                     28-Jun-2007 Fixed compiler warnings
                      17-Jun-2006 Modified init_keyboard_dialog()
 
     Copyright (C) 2005-2007 Marvin Raaijmakers
@@ -47,9 +51,10 @@ History            : 28-Jun-2007 Fixed compiler warnings
 #define SORTID_FIRSTCOL	(0)
 
 
-static void keyboard_treeview_append_keyboard (	GtkListStore	*list_store,
-						char		*manufacturer,
-						char		*model         );
+static void keyboard_treeview_append_keyboard (GtkTreeStore *treestore,
+                                               char         *model,
+                                               GtkTreeIter  *manu_iter,
+                                               GtkTreeIter  *model_iter);
 static void tree_view_add_new_text_column (	GtkTreeView	*tree_view,
 						char		*column_title,
 						int		column_pos       );
@@ -114,7 +119,7 @@ description:
 {
 	GtkWidget *msg_dialog;
 
-	msg_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, msg_type, GTK_BUTTONS_OK, message);
+	msg_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, msg_type, GTK_BUTTONS_OK, message);
 	/*
 	 * Block the interaction with the other windows, so that
 	 * the user can only use this window
@@ -211,71 +216,82 @@ description:
 
 
 void
-keyboard_treeview_append_keyboard (	GtkListStore	*list_store,
-					char		*manufacturer,
-					char		*model         )
+keyboard_treeview_append_keyboard (GtkTreeStore *treestore,
+                                   char         *model,
+                                   GtkTreeIter  *manu_iter,
+                                   GtkTreeIter  *model_iter)
 /*
 input:
-	manufacturer	- The name of the keyboards manufacturer
-	model		- The model of the keyboard
+	manufacturer - The name of the keyboards manufacturer
+	model        - The model of the keyboard
 output:
-	list_store	- The GtkListStore to add the new keyboard to
+	list_store   - The GtkListStore to add the new keyboard to
+	iter         - Will contain the iter of the new row
 returns:
 	-
 description:
 	This function appends a keyboard to list_store.
 */
 {
-	GtkTreeIter	iter;
-	
-	gtk_list_store_append (list_store, &iter);
-	gtk_list_store_set (	list_store, &iter,
-				COL_KEYBOARD_MANUFACTURER, manufacturer,
-				COL_KEYBOARD_MODEL, model,
-				-1 );
+	gtk_tree_store_append (treestore, model_iter, manu_iter);
+	gtk_tree_store_set (treestore, model_iter, 0, model, -1);
 }
 
 
 void
-reload_keyboard_treeview (	GtkWidget	*this_widget,
-				KTKeyboardList	*keyboard_list      )
+reload_keyboard_treeview (GtkTreeView     *keyboard_treeview,
+                          KTKeyboardList  *keyboard_list,
+                          KTKeyboardModel *select_model)
 /*
 input:
-	this_widget		- A widget in the same tree as the widget named "key_treeview"
-	keyboard_list		- List containing keyboards
+	keyboard_list - List containing keyboards
+	select_model  - Points to the model (that is in 'keyboard_list') that should be
+	                selected
 output:
-	-
+	keyboard_treeview - The contents of the tree store will be reloaded with
+	                    information from 'keyboard_list'.
 returns:
 	-
 description:
-	This function deletes all rows of a treeview named "keyboard_treeview", which is
-	in the same tree as this_widget, and appends all keyboard names, which it will
-	read form keyboard_list, to the list.
+	This function deletes all rows of the tree store of 'keyboard_treeview', which
+	is in the same tree as this_widget, and appends all keyboard names, which it
+	will read form keyboard_list, to the list.
 */
 {
-	KTKeyboardManufacturer	*manufacturer;
-	KTKeyboardModel		*model;
-	GtkTreeView		*keyboard_treeview;
-	GtkListStore		*list_store;
+	KTKeyboardManufacturer *manufacturer;
+	KTKeyboardModel *model;
+	GtkTreeStore *treestore;
+	GtkTreeIter model_iter, manu_iter;
+	GtkTreePath *selection_path;
 	
-	keyboard_treeview = GTK_TREE_VIEW( lookup_widget(this_widget, "keyboard_treeview") );
-	list_store = GTK_LIST_STORE( gtk_tree_view_get_model(keyboard_treeview) );
+	treestore = GTK_TREE_STORE( gtk_tree_view_get_model(keyboard_treeview) );
+	
+	gtk_tree_store_clear (treestore); /* Clear the whole treeview */
 	
-	gtk_list_store_clear (list_store); /* Clear the whole treeview */
-
 	/* Fill the treeview */
 	for (manufacturer = keyboard_list->head;
 	     manufacturer != NULL;
 	     manufacturer = manufacturer->next)
 	{
+		gtk_tree_store_append (treestore, &manu_iter, NULL);
+		gtk_tree_store_set (treestore, &manu_iter, 0, manufacturer->name, -1);
+		
 		for (model = manufacturer->model_list.head;
 		     model != NULL;
 		     model = model->next)
 		{
 			/* Add a new row to the model */
-			keyboard_treeview_append_keyboard (	list_store,
-								manufacturer->name,
-								model->name );
+			keyboard_treeview_append_keyboard (treestore, model->name, &manu_iter, &model_iter);
+			if (select_model == model)
+			{
+				/* Get the path of the selection */
+				selection_path = gtk_tree_model_get_path(GTK_TREE_MODEL(treestore), &model_iter);
+				gtk_tree_view_expand_to_path (keyboard_treeview, selection_path);
+				/* Let the view scroll to the selection */
+				gtk_tree_view_scroll_to_cell (keyboard_treeview, selection_path, NULL, TRUE, 0.5, 0);
+				/* Select the new row */
+				gtk_tree_selection_select_iter (gtk_tree_view_get_selection(keyboard_treeview), &model_iter);
+			}
 		}
 	}
 }
@@ -283,49 +299,53 @@ description:
 
 
 void
-init_keyboard_treeview (GtkWidget *this_widget)
+init_keyboard_treeview (GtkTreeView *keyboard_treeview)
 /*
 input:
-	this_widget		- A widget in the same tree as the widget named
-				  "keyboard_treeview"
-output:
 	-
+output:
+	keyboard_treeview - Columns, a selection handler and a cell renderer will
+	                    added to the treeview.
 returns:
 	-
-description:
-	This function initializes a treeview named "keyboard_treeview", which is in
-	the same tree as this_widget.
 */
 {
-	GtkTreeView		*keyboard_treeview;
-	GtkTreeSelection	*selection;
-	GtkListStore		*list_store;
+	GtkTreeSelection  *selection;
+	GtkTreeViewColumn *col;
+	GtkCellRenderer   *renderer;
 		
-	keyboard_treeview = GTK_TREE_VIEW( lookup_widget(this_widget, "keyboard_treeview") );
 	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW(keyboard_treeview) );
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 	
-	/************ Create the model ************/
-	list_store = gtk_list_store_new(NUM_COLS_KEYBOARD_LIST, G_TYPE_STRING, G_TYPE_STRING);
-	gtk_tree_view_set_model (keyboard_treeview, GTK_TREE_MODEL(list_store));
-	g_object_unref (list_store);
+	/* Set the signal handler */
+	gtk_tree_selection_set_select_function (selection, on_keyboard_treeview_selection, NULL, NULL);
 	
-	/************ Create the view ************/
-	tree_view_add_new_text_column (keyboard_treeview, _("Manufacturer"), COL_KEYBOARD_MANUFACTURER);
-	tree_view_add_new_text_column (keyboard_treeview, _("Model"), COL_KEYBOARD_MODEL);
+	col = gtk_tree_view_column_new();
+	/* pack tree view column into tree view */
+	gtk_tree_view_append_column (keyboard_treeview, col);
+	renderer = gtk_cell_renderer_text_new();
+	/* pack cell renderer into tree view column */
+	gtk_tree_view_column_pack_start (col, renderer, TRUE);
+	/* connect 'text' property of the cell renderer to
+	*  model column that contains the model/manufacturer name */
+	gtk_tree_view_column_add_attribute (col, renderer, "text", 0);
+	
+	gtk_tree_view_set_model (keyboard_treeview, GTK_TREE_MODEL(gtk_tree_store_new (1, G_TYPE_STRING)));
 }
 
 
 
 GtkWidget
-*init_keyboard_dialog (	KTKeyboardName		*current_keyboard_name,
-			KTKeyboardList		*keyboard_list )
+*init_keyboard_dialog (KTKeyboardName *current_keyboard_name,
+                       KTKeyboardList *keyboard_list,
+                       KTGUI_type     *gui)
 /*
 Input:
-	current_keyboard_name	- The name of the current keyboard
-	keyboard_list		-
+	current_keyboard_name - The name of the current keyboard
+	keyboard_list         - The list of keyboards
 Output:
-	-
+	gui - Its members will be initialized and will point to the widgets in
+	      the created dialog.
 Returns:
 	The address of the created and initialized dialog.
 Description:
@@ -333,19 +353,20 @@ Description:
 	data (readen from the input variables) to its widgets.
 */
 {
-	GtkWidget *keyboard_dialog;
-
-	/*
-	 * Add the installation directory of the pixmaps to the location where the program
+	/* Add the installation directory of the pixmaps to the location where the program
 	 * should look for its pixmaps
 	 */
 	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
 	add_pixmap_directory ("../pixmaps");
 	
-	keyboard_dialog = create_keyboard_dialog();
-	init_keyboard_treeview (keyboard_dialog);
-	reload_current_keyboard (keyboard_dialog, current_keyboard_name);
-	reload_keyboard_treeview (keyboard_dialog, keyboard_list);
+	gui->main_window = create_keyboard_dialog();
+	gui->keyboard_treeview = GTK_TREE_VIEW( lookup_widget(gui->main_window, "keyboard_treeview") );
+	gui->ok_button = GTK_BUTTON( lookup_widget(gui->main_window, "ok_button") );
 	
-	return keyboard_dialog;
+	init_keyboard_treeview (gui->keyboard_treeview);
+	reload_current_keyboard (gui->main_window, current_keyboard_name);
+	reload_keyboard_treeview (gui->keyboard_treeview, keyboard_list,
+	                          find_keyboard_model(current_keyboard_name, keyboard_list));
+	
+	return gui->main_window;
 }
