@@ -2,7 +2,7 @@
 Name               : set_data.h
 Author             : Marvin Raaijmakers
 Description        : Header file of set_data.c
-Date of last change: 28-Jun-2007
+Date of last change: 13-Jul-2007
 History            : 28-Jun-2007 Fixed compiler warnings
 
     Copyright (C) 2005-2007 Marvin Raaijmakers
@@ -29,6 +29,7 @@ History            : 28-Jun-2007 Fixed compiler warnings
 
 #include <gtk/gtk.h>
 
+#include "gui.h"
 #include <keytouch-keyboard.h>
 
 enum {
@@ -42,9 +43,9 @@ extern char *get_filename_from_user (	char		*title,
 					GtkWindow	*parent_window   );
 extern void msg_box (	gchar		*message,
 			GtkMessageType	msg_type       );
-extern GtkWidget *init_keyboard_dialog (	KTKeyboardName		*current_keyboard_name,
-						KTKeyboardList		*keyboard_list          );
-extern void reload_keyboard_treeview (	GtkWidget	*this_widget,
-					KTKeyboardList	*keyboard_list  );
+extern GtkWidget *init_keyboard_dialog (KTKeyboardName *current_keyboard_name,
+                                        KTKeyboardList *keyboard_list,
+                                        KTGUI_type     *gui);
+extern void reload_keyboard_treeview (GtkTreeView *keyboard_treeview, KTKeyboardList *keyboard_list, KTKeyboardModel *select_model);
 
 #endif
