@@ -2,15 +2,17 @@
 Name               : check_desktop.c
 Author             : Marvin Raaijmakers
 Description        : Identifies the currently running desktop environment
-Date of last change: 24-Sep-2006
-History            : 24-Sep-2006 is_gnome_desktop() now also returns TRUE when
+Date of last change: 05-Oct-2007
+History            : 05-Oct-2007 Change is_kde_desktop():
+                                  now checks if env var KDE_FULL_SESSION is set
+                     24-Sep-2006 is_gnome_desktop() now also returns TRUE when
                                  the application "gnome-panel" is running.
                      03-Jun-2006 is_gnome_desktop() now checks if the
                                  GNOME_DESKTOP_SESSION_ID env variable exists
                      26-Jan-2006 Changed "dcop kdesktop >> /dev/null" to
                                  "dcop kdesktop &> /dev/null"
 
-    Copyright (C) 2004-2006 Marvin Raaijmakers
+    Copyright (C) 2004-2007 Marvin Raaijmakers
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -51,11 +53,7 @@ description:
 	This function checks if KDE is running.
 */
 {
-	if ( WEXITSTATUS(system("dcop kdesktop &> /dev/null")) ) /* If KDE is NOT running */
-	{
-		return (FALSE);
-	}
-	return (TRUE);
+	return getenv("KDE_FULL_SESSION") != NULL;
 }
 
 
@@ -72,7 +70,7 @@ description:
 	This function checks if GNOME is running.
 */
 {
-	return (getenv("GNOME_DESKTOP_SESSION_ID") != NULL || !WEXITSTATUS( system("ps -C \"gnome-panel\" &> /dev/null") ));
+	return getenv("GNOME_DESKTOP_SESSION_ID") != NULL || !WEXITSTATUS( system("ps -C \"gnome-panel\" &> /dev/null") );
 }
 
 
